'use strict';

const {operator} = require('putout');
const {compare} = operator;

const {
    entries,
    keys,
} = Object;

module.exports.report = () => 'Avoid duplicate functions';

module.exports.fix = ({path, duplicatePath}) => {
    const {name} = path.get('id').node;
    const {name: duplicateName} = duplicatePath.get('id').node;
    
    path.scope.rename(duplicateName, name);
    duplicatePath.remove();
};

module.exports.traverse = ({push}) => ({
    Program({scope}) {
        const {bindings} = scope;
        const names = keys(bindings);
        const bodies = [];
        
        for (const [i] of entries(names)) {
            for (const [j] of entries(names)) {
                const fn1 = names[i];
                const fn2 = names[j];
                
                const fn1Path = bindings[fn1].path;
                const fn2Path = bindings[fn2].path;
                
                if (!fn1Path.isVariableDeclarator())
                    continue;
                
                if (!fn2Path.isVariableDeclarator())
                    continue;
                
                const initPath1 = fn1Path.get('init');
                const initPath2 = fn2Path.get('init');
                
                if (!initPath1.isFunction())
                    continue;
                
                if (!initPath2.isFunction())
                    continue;
                
                if (initPath1.node.type !== initPath2.node.type)
                    continue;
                
                if (compare(initPath1, initPath2))
                    bodies.push({path: fn1Path, duplicatePath: fn2Path});
            }
        }
        
        for (const {path, duplicatePath} of bodies) {
            if (!path.node || !duplicatePath.node)
                continue;
            
            if (path === duplicatePath)
                continue;
            
            push({
                path,
                duplicatePath,
            });
        }
    },
});

